
include { BOWTIE2_ALIGNMENT } from '../../modules/local/hicstuff/align_bowtie2'
include { FRAGMENT_ENZYME } from '../../modules/local/hicstuff/fragment_enzyme'
include { BAM2PAIRS } from '../../modules/local/hicstuff/bam2pairs'
include { BUILD_MATRIX } from '../../modules/local/hicstuff/build_matrix'
include { BUILD_MATRIX_COOL } from '../../modules/local/hicstuff/build_matrix_cool'
include { FILTER_EVENT } from '../../modules/local/hicstuff/filter_event'
include { DISTANCE_LAW } from '../../modules/local/hicstuff/distance_law'
include { FILTER_PCR } from '../../modules/local/hicstuff/filter_pcr'
include { FILTER_PCR_DUP } from './filter_pcr_dup'
include { HIC_PLOT_MATRIX} from '../../modules/local/hicexplorer/hicPlotMatrix'
include { ITERALIGN } from '../../modules/local/hicstuff/iteralign'

// Paired-end to Single-end
def pairToSingle(row, mates) {
    def meta = row[0].clone()
    meta.single_end = true
    meta.mates = mates
    if (mates == "R1") {
        return [meta, [ row[1][0]] ]
    }else if (mates == "R2"){
        return [meta, [ row[1][1]] ]
    }
}

// Single-end to Paired-end
def singleToPair(row){
    def meta = row[0].clone()
    meta.remove('mates')
    meta.single_end = false
    return [ meta, row[1] ]
}

workflow HICSTUFF {

    take:
    reads
    index
    ligation_site
    digestion
    fasta
    chromosome_size

    main:
    ch_versions = Channel.empty()

    // Align each mates separetly and add mates information in [meta]
    ch_reads_r1 = reads.map{ it -> pairToSingle(it,"R1") }
    ch_reads_r2 = reads.map{ it -> pairToSingle(it,"R2") }
    ch_reads = ch_reads_r1.concat(ch_reads_r2)

    if (params.iteralign && params.cutsite) {
        error "Error: iteralign and cutsite can't both be true at the same time! Set one of them false"
    }
    else if (params.iteralign){
        ITERALIGN(
            ch_reads,
            index.collect()
        )
        ch_bam = ITERALIGN.out.bam
        ch_versions = ch_versions.mix(ITERALIGN.out.versions)
    }
    else {
        BOWTIE2_ALIGNMENT(
        ch_reads,
        index.collect()
        )
        ch_bam = BOWTIE2_ALIGNMENT.out.bam
        ch_versions = ch_versions.mix(BOWTIE2_ALIGNMENT.out.versions)
    }


    FRAGMENT_ENZYME(
        digestion,
        fasta
    )
    ch_versions = ch_versions.mix(FRAGMENT_ENZYME.out.versions)

    if (params.filter_pcr && params.filter_pcr_picard ){
        error "Error: filter_pcr and filter_pcr_picard can't both be true at the same time! Set one of them false in the config file"
    }
    else if (params.filter_pcr_picard){
        FILTER_PCR_DUP(
            ch_bam,
            fasta,
            index
        )
        FILTER_PCR_DUP.out.bam.set{ new_ch_bam }
        ch_versions = ch_versions.mix(FILTER_PCR_DUP.out.versions)
    }
    else {
        ch_bam.combine(ch_bam)
        .map {
            meta1, bam1, meta2, bam2 ->
                meta1.id == meta2.id && meta1.chunk == meta2.chunk && meta1.mates == "R1" && meta2.mates == "R2" ? [ meta1,  bam1,  meta2, bam2 ] : null
        }.set{ new_ch_bam }

    }

    BAM2PAIRS(
        new_ch_bam,
        FRAGMENT_ENZYME.out.info_contigs.collect(),
        digestion,
        fasta.collect()
    )
    BAM2PAIRS.out.idx_pairs.set{ ch_idx_pairs }
    BAM2PAIRS.out.valid_pairs.set{ ch_valid_pairs }

    if( params.filter_event ){
        FILTER_EVENT(
            ch_idx_pairs
        )
        FILTER_EVENT.out.idx_pairs_filtered.set{ ch_idx_pairs }
    }

    if ( params.distance_law ){
        DISTANCE_LAW(
            ch_idx_pairs,
            FRAGMENT_ENZYME.out.fragments_list.collect()
        )
    }

    if (params.filter_pcr && params.filter_pcr_picard ){
        error "Error: filter_pcr and filter_pcr_picard can't both be true at the same time! Set one of them false in the config file"
    }
    else if ( params.filter_pcr ){
        FILTER_PCR(
            ch_idx_pairs
        )
        FILTER_PCR.out.idx_pairs_pcrfree.set{ ch_idx_pairs }
    }

    ch_idx_pairs.combine(ch_valid_pairs)
    .map {
        meta1, idx, meta2, pairs ->
            meta1 == meta2 ? [ meta1, idx, pairs ] : null
    }.set{ ch_pairs }

    BUILD_MATRIX(
        ch_idx_pairs,
        FRAGMENT_ENZYME.out.fragments_list.collect()
    )

    BUILD_MATRIX_COOL(
        ch_idx_pairs,
        FRAGMENT_ENZYME.out.fragments_list.collect()
    )

    emit:
    pairs = ch_pairs
    versions = ch_versions
}
